/*
 * SocketStream.java
 *
 * Copyright (C) 2004 Peter Graves
 * $Id: SocketStream.java,v 1.3 2004/10/13 00:22:19 piso Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.armedbear.lisp;

import java.net.Socket;

public final class SocketStream extends TwoWayStream
{
    private final Socket socket;

    public SocketStream(Socket socket, Stream in, Stream out)
    {
	super(in, out);
        this.socket = socket;
    }

    public LispObject typeOf()
    {
        return Symbol.SOCKET_STREAM;
    }

    public LispObject classOf()
    {
        return BuiltInClass.SOCKET_STREAM;
    }

    public LispObject typep(LispObject type) throws ConditionThrowable
    {
        if (type == Symbol.SOCKET_STREAM)
            return T;
        if (type == BuiltInClass.SOCKET_STREAM)
            return T;
        return super.typep(type);
    }

    public LispObject close(LispObject abort) throws ConditionThrowable
    {
	try {
	    socket.close();
	    setOpen(false);
	    return T;
	} catch (Exception e) {
	    return signal(new LispError(e.getMessage()));
	}
    }

    public String toString()
    {
        return unreadableString("SOCKET-STREAM");
    }
}
